###########################################################################
#
# PhotafMetadataJSONPlugin.pm -- A plugin for audio structured JSON files
#
# A component of the Greenstone digital library software
# from the New Zealand Digital Library Project at the 
# University of Waikato, New Zealand.
#
# Copyright 2013 New Zealand Digital Library Project
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
###########################################################################

package PhotafMetadataJSONPlugin;


use BasePlugin;
use MetadataRead;

use strict;
no strict 'refs';
use multiread;

use Encode;
use JSON;

# methods with identical signatures take precedence in the order given in the ISA list.
sub BEGIN {
    @PhotafMetadataJSONPlugin::ISA = ('MetadataRead', 'BasePlugin');
}


my $arguments = [
      { 'name' => "process_exp",
	'desc' => "{BasePlugin.process_exp}",
	'type' => "regexp",
	'reqd' => "no",
	'deft' => &get_default_process_exp() }

];


my $options = { 'name'     => "PhotafMetadataJSONPlugin",
		'desc'     => "{PhotafMetadataJSONPlugin.desc}",
		'abstract' => "no",
		'inherits' => "yes",
		'args'     => $arguments };


sub new
{
    my ($class) = shift (@_);
    my ($pluginlist,$inputargs,$hashArgOptLists) = @_;
    push(@$pluginlist, $class);

    push(@{$hashArgOptLists->{"ArgList"}},@{$arguments});
    push(@{$hashArgOptLists->{"OptList"}},$options);

    my $self = new BasePlugin($pluginlist, $inputargs, $hashArgOptLists);

    return bless $self, $class;
}


sub get_default_process_exp
{
    return q^(?i)PhotafPanoramaConfig\.cfg$^;
}

sub file_block_read {
    my $self = shift (@_);
    my ($pluginfo, $base_dir, $file, $block_hash, $metadata, $gli) = @_;

    my ($filename_full_path, $filename_no_path) = &util::get_full_filenames($base_dir, $file);

    if (!-f $filename_full_path || !$self->can_process_this_file($filename_full_path)) {
	return undef; # can't recognise
    }

    # set this so we know this is a metadata file - needed for incremental 
    # build
    # if this file changes, then we need to reimport everything
    $block_hash->{'metadata_files'}->{$filename_full_path} = 1;

    return 1;
}

sub metadata_read
{
    my $self = shift (@_);
    my ($pluginfo, $base_dir, $file, $block_hash, 
	$extrametakeys, $extrametadata, $extrametafile,
	$processor, $gli, $aux) = @_;

    # Read metadata from JSON files
    my $filename = &util::filename_cat($base_dir, $file);

#    if ($filename !~ /\.js$/ || !-f $filename) {

    if (!-f $filename || !$self->can_process_this_file($filename)) {

	return undef;
    }
    print STDERR "\n<Processing n='$file' p='PhotafMetadataJSONPlugin'>\n" if ($gli);
    print STDERR "PhotafMetadataJSONPlugin: processing $file\n" if ($self->{'verbosity'}) > 1;

    my $outhandle = $self->{'outhandle'};
    my $failhandle = $self->{'failhandle'};

    # add the file to the block list so that it won't be processed in read, as we will do all we can with it here
    &util::block_filename($block_hash,$filename);


    # Read the JSON file to get the metadata
    my $json_file_content;
    open(JSON_FILE, "$filename");
    my $json_file_reader = new multiread();
    $json_file_reader->set_handle('PhotafMetadataJSONPlugin::JSON_FILE');
    $json_file_reader->read_file(\$json_file_content);

    # Would be nice if PhotafMetadataJSONPlugin was extended to support a minus
    # option to choose the character encoding the JSON file is in 
    # For now we will assume it is always in UTF8
    $json_file_content = decode("utf8",$json_file_content);

    close(JSON_FILE);

    # Split the file into lines and read the first line (contains the metadata names)
    $json_file_content =~ s/\r/\n/g;  # Handle non-Unix line endings
    $json_file_content =~ s/\n+/\n/g;
#    my @json_file_lines = split(/;/s, $json_file_content);

#    my $matching_file = $file;
#    $matching_filename =~ s/\.js/\.{ogg,mp3,wav}/;


    my $json_metadata = {};

    $json_file_content =~ s/^\{//;
    $json_file_content =~ s/\}$//;

    my @json_pairs_conjoined = split(/,/,$json_file_content);


    # Read each line of the file and assign the metadata appropriately
    foreach my $json_pair_str (@json_pairs_conjoined) {

    
	my ($metaname,$metaval) = ($json_pair_str =~ m/\"(.+?)\":\"?(.+?)\"?$/);

	$metaname =~ s/^m_//;
	$json_metadata->{$metaname} = [$metaval];
	$json_metadata->{"photaf.".$metaname} = [$metaval];
	
    }
    
    # Associate the metadata with the companion jpg image
    my $json_filename = "PhotafPanoramaPicHD.jpg";
    $json_filename = &util::filename_to_regex($json_filename);

    $extrametadata->{$json_filename} = $json_metadata;
    push(@$extrametakeys, $json_filename);

    # record which file the metadata came from 
    if (!defined $extrametafile->{$json_filename}) {
	$extrametafile->{$json_filename} = {};
    }
    # maps the file to full path
    $extrametafile->{$json_filename}->{$file} = $filename;

    return 1; 
}

sub print_error
{
    my $self = shift(@_);
    my ($outhandle, $failhandle, $gli, $file, $error) = @_;

    print $outhandle "PhotafMetadataJSONPlugin Error: $file: $error\n";
    print $failhandle "PhotafMetadataJSONPlugin Error: $file: $error\n";
    print STDERR "<ProcessingError n='$file' r='$error'/>\n" if ($gli);
}
1;
